@@ -6,7 +6,7 @@ use CHI::Util qw(require_dynamic);
 use strict;
 use warnings;
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 our $Logger = CHI::NullLogger->new();    ## no critic
 
@@ -32,7 +32,7 @@ sub new {
       unless defined $driver_class;
     require_dynamic($driver_class);
 
-    return $driver_class->new( \%params );
+    return $driver_class->new(%params);
 }
 
 1;
@@ -482,101 +482,9 @@ L<CHI::Driver::CacheCache|CHI::Driver::CacheCache> - CHI wrapper for Cache::Cach
 
 =for readme stop
 
-=head1 IMPLEMENTING NEW DRIVERS
+=head1 DEVELOPING NEW DRIVERS
 
-To implement a new driver, create a new subclass of CHI::Driver and implement an
-appropriate subset of the methods listed below. We recommend that you call your subclass
-CHI::Driver::I<something> so that users can create it simply by passing I<something> to the
-C<driver> option of CHI-E<gt>new.
-
-The easiest way to start is to look at, subclass, or copy existing drivers, such as the
-L<CHI::Driver::Memory|CHI::Driver::Memory> and L<CHI::Driver::File|CHI::Driver::File>
-included with this distribution.
-
-All cache handles have an assigned namespace that you can access with
-C<$self-E<gt>namespace>. You should use the namespace to partition your data store.
-
-=head2 Required methods
-
-The following methods have no default implementation, and MUST be defined by your subclass:
-
-=over
-
-=item store ( $self, $key, $data, $expires_at, $options )
-
-Associate I<$data> with I<$key> in the namespace, overwriting any existing entry.  Called
-by L</set>. I<$data> will contain any necessary metadata, including expiration options, so
-you can just store it as a single block.
-
-The I<$expires_at> epoch value is provided in case you are working with an existing
-cache implementation (like memcached) that also has an interest in storing the
-expiration time for its own purposes. Normally, you can ignore this.
-
-=item fetch ( $self, $key )
-
-Returns the data associated with I<$key> in the namespace. Called by L</"set">. The main
-CHI::Driver superclass will take care of extracting out metadata like expiration options
-and determining if the value has expired.
-
-=item remove ( $self, $key )
-
-Remove the data associated with the I<$key> in the namespace.
-
-=back
-
-=head2 Overridable methods
-
-The following methods have a default implementation, but MAY be overriden by your subclass:
-
-=over
-
-=item new ( %options )
-
-Override the constructor if you want to process any options specific to your driver.
-
-=item clear ( $self )
-
-Override this if you want to provide an efficient method of clearing a namespace.
-The default implementation will iterate over all keys and call L</remove> for each.
-
-=item _serialize ( $value )
-
-=item _deserialize ( $serialized_value )
-
-Override these if you want to change the serialization method used for references. The
-default is Storable freeze/thaw.
-
-=item fetch_multi_arrayref ( $keys )
-
-Override this if you want to efficiently process multiple fetches. Return an array
-reference of data or undef corresponding to I<$keys>. The default will iterate over
-I<$keys> and call fetch for each.
-
-=item store_multi ( $key_data, $options )
-
-Override this if you want to efficiently process multiple stores. I<$key_data> is a hash
-of keys and data that should be stored. The default will iterate over I<$key_data> and
-call store for each pair.
-
-=back
-
-=head2 Optional methods
-
-The following methods have no default implementation, and MAY be defined by your subclass,
-but are not required for basic cache operations.
-
-=over
-
-=item get_keys ( $self )
-
-Return all keys in the namespace. It is acceptable to include or omit expired keys.
-
-=item get_namespaces ( $self )
-
-Return namespaces associated with the cache. It is acceptable to include or omit namespaces
-with no valid keys.
-
-=back
+See L<CHI::Driver::Development|CHI::Driver::Development> for information on developing new drivers.
 
 =head1 LOGGING
 
