@@ -6,6 +6,14 @@ use base qw(CHI::t::Driver);
 
 my $testaddr = "127.0.0.1:11211";
 
+# Making this internal only for now, since test success depends on presence and configuration
+# of an external server.
+#
+# When CHI::Driver::Memcached is pulled out into a separate distribution, it'll make
+# sense to test externally again.
+#
+sub internal_only { 1 }
+
 sub required_modules {
     return { 'Cache::Memcached' => undef, 'IO::Socket::INET' => undef };
 }
@@ -55,7 +63,17 @@ sub set_standard_keys_and_values {
     return ( $keys, $values );
 }
 
-sub supports_clear    { 0 }
-sub supports_get_keys { 0 }
+sub test_get_keys : Test(1) {
+    my $self = shift;
+
+    # Make sure we get a 'not supported' error with regular memcached driver
+    my $cache =
+      $self->SUPER::new_cache( driver => 'Memcached', servers => [$testaddr] );
+    throws_ok(
+        sub { $cache->get_keys() },
+        qr/not supported/,
+        "get_keys not supported"
+    );
+}
 
 1;
